@@ -18,10 +18,10 @@ class Causalml(CausalEstimator):
 
         # Check the backdoor variables being used
         self.logger.debug("Back-door variables used:" +
-                          ",".join(self._target_estimand.backdoor_variables))
+                          ",".join(self._target_estimand.get_backdoor_variables()))
         
         # Add the observed confounders and one hot encode the categorical variables
-        self._observed_common_causes_names = self._target_estimand.backdoor_variables
+        self._observed_common_causes_names = self._target_estimand.get_backdoor_variables()
         if self._observed_common_causes_names:
             # Get the data of the unobserved confounders
             self._observed_common_causes = self._data[self._observed_common_causes_names]
@@ -85,11 +85,20 @@ class Causalml(CausalEstimator):
         matched_args = {
             arg: func_args[arg] for arg in func_args.keys() if arg in arg_names 
         }
+        print(matched_args)
         value_tuple = self.estimator.estimate_ate(**matched_args) 
 
+        # For CATEs
+        arg_names = inspect.getfullargspec(self.estimator.fit_predict)[0]
+        matched_args = {
+            arg: func_args[arg] for arg in func_args.keys() if arg in arg_names 
+        }
+        cate_estimates = self.estimator.fit_predict(**matched_args) 
+
         estimate = CausalEstimate(estimate=value_tuple[0],
                                   target_estimand=self._target_estimand,
                                   realized_estimand_expr=self.symbolic_estimator,
+                                  cate_estimates = cate_estimates,
                                   effect_intervals=(value_tuple[1],value_tuple[2]),
                                   _estimator_object=self.estimator)
 
@@ -100,7 +109,7 @@ class Causalml(CausalEstimator):
         expr = "b: " + ",".join(estimand.outcome_variable) + "~"
         # TODO we are conditioning on a postive treatment
         # TODO create an expression corresponding to each estimator used
-        var_list = estimand.treatment_variable + estimand.backdoor_variables
+        var_list = estimand.treatment_variable + estimand.get_backdoor_variables()
         expr += "+".join(var_list)
         return expr
  
