@@ -11,7 +11,6 @@ import dowhy.utils.cli_helpers as cli
 from dowhy.causal_estimator import CausalEstimate
 from dowhy.causal_graph import CausalGraph
 from dowhy.causal_identifier import CausalIdentifier
-
 from dowhy.utils.api import parse_state
 
 init_printing()  # To display symbolic math symbols
@@ -51,6 +50,7 @@ class CausalModel:
         :param estimand_type: the type of estimand requested (currently only "nonparametric-ate" is supported). In the future, may support other specific parametric forms of identification.
         :param proceed_when_unidentifiable: does the identification proceed by ignoring potential unobserved confounders. Binary flag.
         :param missing_nodes_as_confounders: Binary flag indicating whether variables in the dataframe that are not included in the causal graph, should be  automatically included as confounder nodes.
+        :param `**kwargs`: More optional parameters that can be passed to the causal model. Currently supported params: "logging_level" to indicate the level of logging needed. Possible values are logging.CRITICAL, logging.ERROR, logging.INFO, logging.WARNING,and logging.DEBUG. Default is logging.INFO.
         :returns: an instance of CausalModel class
 
         """
@@ -126,7 +126,7 @@ class CausalModel:
         self._other_variables = kwargs
         self.summary()
 
-    def identify_effect(self, proceed_when_unidentifiable=None):
+    def identify_effect(self, estimand_type=None, proceed_when_unidentifiable=None):
         """Identify the causal effect to be estimated, using properties of the causal graph.
 
         :param proceed_when_unidentifiable: Binary flag indicating whether identification should proceed in the presence of (potential) unobserved confounders.
@@ -135,9 +135,11 @@ class CausalModel:
         """
         if proceed_when_unidentifiable is None:
             proceed_when_unidentifiable = self._proceed_when_unidentifiable
+        if estimand_type is None:
+            estimand_type = self._estimand_type
 
         self.identifier = CausalIdentifier(self._graph,
-                                           self._estimand_type,
+                                           estimand_type,
                                            proceed_when_unidentifiable=proceed_when_unidentifiable)
         identified_estimand = self.identifier.identify_effect()
 
@@ -157,11 +159,11 @@ class CausalModel:
             * Propensity Score Stratification: "backdoor.propensity_score_stratification"
             * Propensity Score-based Inverse Weighting: "backdoor.propensity_score_weighting"
             * Linear Regression: "backdoor.linear_regression"
+            * Generalized Linear Models (e.g., logistic regression): "backdoor.generalized_linear_model"
             * Instrumental Variables: "iv.instrumental_variable"
             * Regression Discontinuity: "iv.regression_discontinuity"
 
-        In addition, you can directly call any of the EconML estimation methods. The convention is "backdoor.econml.path-to-estimator-class". For example, for the double machine learning estimator ("DMLCateEstimator" class) that is located inside "dml" module of EconML, you can use the method name, "backdoor.econml.dml.DMLCateEstimator".
-
+        In addition, you can directly call any of the EconML estimation methods. The convention is "backdoor.econml.path-to-estimator-class". For example, for the double machine learning estimator ("DMLCateEstimator" class) that is located inside "dml" module of EconML, you can use the method name, "backdoor.econml.dml.DMLCateEstimator". CausalML estimators can also be called. See `this demo notebook <https://microsoft.github.io/dowhy/example_notebooks/dowhy-conditional-treatment-effects.html>`_.
 
 
         :param identified_estimand: a probability expression
@@ -208,12 +210,12 @@ class CausalModel:
                     # Define the third-party estimation method to be used
                     method_params["_" + third_party_estimator_package + "_methodname"] = estimator_name
             else: # For older dowhy methods
-                # Process the dowhy estimators 
+                # Process the dowhy estimators
                 causal_estimator_class = causal_estimators.get_class_object(estimator_name + "_estimator")
 
         # Check if estimator's target estimand is identified
         if identified_estimand.estimands[identifier_name] is None:
-            self.logger.warning("No valid identified estimand for using instrumental variables method")
+            self.logger.warning("No valid identified estimand available.")
             estimate = CausalEstimate(None, None, None)
         else:
             causal_estimator = causal_estimator_class(
@@ -298,7 +300,7 @@ class CausalModel:
         :param estimand: target estimand, an instance of the IdentifiedEstimand class (typically, the output of identify_effect)
         :param estimate: estimate to be refuted, an instance of the CausalEstimate class (typically, the output of estimate_effect)
         :param method_name: name of the refutation method
-        :param **kwargs:  (optional) additional arguments that are passed directly to the refutation method. Can specify a random seed here to ensure reproducible results ('random_seed' parameter). For method-specific parameters, consult the documentation for the specific method. All refutation methods are in the causal_refuters subpackage.
+        :param kwargs:  (optional) additional arguments that are passed directly to the refutation method. Can specify a random seed here to ensure reproducible results ('random_seed' parameter). For method-specific parameters, consult the documentation for the specific method. All refutation methods are in the causal_refuters subpackage.
 
         :returns: an instance of the RefuteResult class
 
@@ -327,10 +329,37 @@ class CausalModel:
         """
         self._graph.view_graph(layout)
 
-    def summary(self):
-        """Print a text summary of the model.
+    def interpret(self, method_name=None, **kwargs):
+        """Interpret the causal model.
+
+        :param method_name: method used for interpreting the model. If None,
+                            then default interpreter is chosen that describes the model summary and shows the associated causal graph.
+        :param kwargs:: Optional parameters that are directly passed to the interpreter method.
 
         :returns: None
 
         """
-        self.logger.info("Model to find the causal effect of treatment {0} on outcome {1}".format(self._treatment, self._outcome))
+        if method_name is None:
+            self.summary(print_to_stdout=True)
+            self.view_model()
+            return
+
+        method_name_arr = parse_state(method_name)
+        import dowhy.interpreters as interpreters
+        for method in method_name_arr:
+            interpreter = interpreters.get_class_object(method)
+            interpreter(self, **kwargs).interpret()
+
+    def summary(self, print_to_stdout=False):
+        """Print a text summary of the model.
+
+        :returns: a string containining the summary
+
+        """
+        summary_text = "Model to find the causal effect of treatment {0} on outcome {1}".format(self._treatment, self._outcome)
+        self.logger.info(summary_text)
+        if print_to_stdout:
+            print(summary_text)
+        return summary_text
+
+
